function Output = quantization_in_Fourier_domain(Input,QL,block_size)
%
% quantization_in_Fourier_domain(I,QL,block_size)
%
% - Input ... an input image matrix
% - QL ... quantization level (%)
% - block_size ... the image will be split into small rectangles, each of
%                  size 'block_size(1) x block_size(2)'
%
% example of use:
%
% output = quantization_in_Fourier_domain(I, 40, [8 8])
%

if ((QL>100) || (QL<0))
    error('Bad argument');
    return;
end
    
% 1. split the image into several blocks
% 2. for each block do
% 3.    apply 2D-DCT
% 3.    apply quantization to DCT coefficients 
% 4.    apply inverse 2D-DCT
% 4. end for
% 5. compose the blocks into the image of the original size


% convert image into matrix
Input = double(Input);

% image size
size_x = size(Input,2);
size_y = size(Input,1);

% number of blocks
count_x = ceil(size_x / block_size(1));
count_y = ceil(size_y / block_size(2));

% prepare the mask matrix and linearize it
linear_mask = zigzag(ones(block_size(2), block_size(1)));
% how many coefficients should be eliminated
limit = floor(block_size(1)*block_size(2)*(1.0 - double(QL)/100));
% eliminate the coefficients
linear_mask(limit:end) = 0; 
% convert the 1D vector back to 
mask = izigzag(linear_mask, block_size(2), block_size(1))

for x_th=1:count_x,       
    for y_th=1:count_y,
        % find the area of the current block
        block_start_x = (x_th-1)*block_size(1) + 1;
        block_start_y = (y_th-1)*block_size(2) + 1;
        % pay attention to the boundary item
        block_end_x = min(block_start_x + block_size(1) - 1,size_x);
        block_end_y = min(block_start_y + block_size(2) - 1,size_y);
        
        % copy the area
        current_block = Input(block_start_y:block_end_y, block_start_x:block_end_x);
        
        % store the real size of the current area
        real_block_size = fliplr(size(current_block));   
    
		% perform DCT
        DCT_coeffs = dct2(current_block);
        
        % test whether the current block has the appropriate size
        % (whether it is/is not on the border)
        if ((block_size(1) ~= real_block_size(1)) || (real_block_size(2) ~= block_size(2)))        
            % linearize the Fourier domain in the zig-zag style.            
            linear = zigzag(DCT_coeffs);
      
            % remove selected Fourier coefficients
            limit = floor((real_block_size(1)*real_block_size(2))*(1.0 - double(QL)/100));
            linear(limit:end) = 0;            
            modified_coeffs = izigzag(linear, real_block_size(2), real_block_size(1));            
        else
            modified_coeffs = DCT_coeffs .* mask;            
        end  
        
        % perform inverse DCT
        current_block = idct2(modified_coeffs);
       
		% update the active area (window)
        Output(block_start_y:block_end_y, block_start_x:block_end_x) = current_block(:,:);
    end
end

Output = dip_image(Output);

end

